/*------------------------------------------------------------------------
 *
 * File:        vfd.c
 *
 * Author:	Suzy Jackson <suzyj@littlefishbicycles.com>
 *
 * Revision:	0.1
 *
 * Commenced:	24 April 2006.
 *
 * Last edit:	24 April 2006.
 *
 * Purpose:     Graphical VFD routines.
 *
 * Description: This file contains a set of useful routines for use with
 *		a Noritake-Itron GU140X16G graphical Vacuum Fluorescent 
 *		Display (VFD).
 *
 *------------------------------------------------------------------------*/

#include <pic.h>
#include "vfd.h"
#include "delay.h"
#include "hardware.h"

/* write a byte to the VFD cmd register */

void
vfd_write_cmd(unsigned char c)
{
	unsigned char busy;
	TRISD = 0xff;
	VFD_RW = 1;		// read
	VFD_RS = 0;		
	do {
		VFD_EN = 1;
		busy = DATA & 0x80;
		VFD_EN = 0;
	} while (busy);
	TRISD = 0x00;
	VFD_RW = 0;		// write
	VFD_RS = 0;		
	DATA = c;
	VFD_EN = 1;
	VFD_EN =0;
}

/* write a byte to the VFD data register */

void
vfd_write_data(unsigned char c)
{
	unsigned char busy;
	TRISD = 0xff;
	VFD_RW = 1;		// read
	VFD_RS = 0;		
	do {
		VFD_EN = 1;
		busy = DATA & 0x80;
		VFD_EN = 0;
	} while (busy);
	TRISD = 0x00;
	VFD_RW = 0;		// write
	VFD_RS = 1;		
	DATA = c;
	VFD_EN = 1;
	VFD_EN =0;
}

/*
 * 	Clear and home the LCD
 */

void
vfd_clear(void)
{
	vfd_write_cmd(0x01);
}

/* write a string of chars to the VFD */

void
vfd_puts(const char * s)
{
	while(*s)
		vfd_write_data(*s++);
}

/*
 * Go to the specified position
 *
 *	xpos = 0-139, ypos = 0-15
 */

void
vfd_goto(unsigned char xpos, unsigned char ypos)
{
	vfd_write_cmd(0xf0);
	vfd_write_data(xpos);	
	vfd_write_data(ypos);	
}

/*
 * Set the VFD font
 *
 *	'A' proportional mini font
 *	'B' 5x7 Katakana font
 *	'C' 10x14 Katakana font 
 *	'b' 5x7 European font
 *	'c' 10x14 European font 
 *	'1' 1 pixel inter-character spacing
 *	'2' 2 pixel inter-character spacing 
 */

void
vfd_font(unsigned char font)
{
	vfd_write_cmd(0xf2);
	vfd_write_data(font);	
}	

/*
 * Set the VFD area
 *
 *	x1, y1 = left top of area
 *	x2, y2 = right bottom of area
 *
 *	Commands:
 *		'I' Invert area
 *		'F' Fill area
 *		'C' Clear area 
 *		'O' Set outline
 *		'o' Clear outline
 *		'H' Write horizontal graphical data with horizontal cursor movement
 *		'V' Write vertical graphical data with vertical cursor movement
 *		'h' Write horizontal graphical data with vertical cursor movement
 *		'v' Write vertical graphical data with horizontal cursor movement
 */

void
vfd_area(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char cmd)
{
	vfd_write_cmd(0xf1);
	vfd_write_data(x1);	
	vfd_write_data(y1);	
	vfd_write_data(x2);	
	vfd_write_data(y2);	
	vfd_write_data(cmd);	
}	
	
/* initialise the VFD - put into 8 bit mode */
void
vfd_init()
{
	VFD_EN = 0;
	vfd_write_cmd (0x30);
	DelayMs (10);
	vfd_write_cmd (0x30);
	DelayMs (10);
	vfd_write_cmd (0x30);
	DelayMs (10);
	vfd_write_cmd (0x30);	// Eight bit mode
	vfd_write_data(0x01);	// Set luminance=75%
	vfd_write_cmd(0x06);
	vfd_write_cmd(0x0c);
	vfd_clear();		// Clear screen
}
